/**
 * Un example d'utilisation de GtkImage qui charge une image
 * depuis un fichier PNG et modifie les pixels de l'image.
 *
 * An example using a GtkImage which loads an image from a PNG
 * file and modifies the pixels of the image.
 *
 * AUTHOR: Guillaume Riviere (C) 2012
 * FILE: main.c
 */

#include <gtk/gtk.h>

#include "gdkpixbuf_accessors.h"

/* Trace de cercle */
void draw_circle_andres (GdkPixbuf *pixbuf, int x_centre, int y_centre, int rayon, guchar R, guchar G, guchar B) ;

/* Callbacks pour le signal de destruction de la fenetre principale */
G_MODULE_EXPORT void on_mainwindow_destroy (GtkObject *object, gpointer user_data) {
  gtk_main_quit () ;  /* Stopper la boucle GTK */
}

/* Fonction principale */
int main (int argc, char *argv[]) {
  GtkBuilder *builder = NULL ;
  GtkWindow  *mainwindow = NULL ;
  GtkImage   *image1 = NULL ;
  GdkPixbuf  *gdk_pixbuf = NULL ;
  int w, h ;
  int x, y ;

  gtk_init (&argc, &argv);

  /* Creer tous les widgets grace au builder GTK */
  builder = gtk_builder_new ();
  gtk_builder_add_from_file (builder, "gtk_image_example_2.24.glade", NULL);
  gtk_builder_connect_signals (builder, NULL);

  /*------------*/

  mainwindow = GTK_WINDOW (gtk_builder_get_object (builder, "mainwindow"));
  image1 = GTK_IMAGE (gtk_builder_get_object (builder, "image1"));

  /* Creer un GdkPixbuf depuis un fichier PNG */
  gdk_pixbuf = gdk_pixbuf_new_from_file ("gtk.png", NULL);
  w = gdk_pixbuf_get_width (gdk_pixbuf) ;
  h = gdk_pixbuf_get_height (gdk_pixbuf) ;

  /* Modifier la valeur d'un pixel */
  gdkpixbuf_set_pixel_color (gdk_pixbuf, 10, 10, 0, 0, 255) ;

  /* Dessiner le carre du tier central de l'image en rouge*/
  for (x=w/3 ; x<2*w/3 ; x++) {
    for (y=h/3 ; y<2*h/3 ; y++) {
      gdkpixbuf_set_pixel_color (gdk_pixbuf, x, y, 255, 0, 0) ;
    }
  }

  /* Dessiner un cercle bleu au centre de l'image */
  draw_circle_andres (gdk_pixbuf, w/2, h/2, 100, 0, 0, 255) ;

  /* Afficher le pixbuf dans un GtkImage */
  gtk_image_set_from_pixbuf (image1, gdk_pixbuf) ;
  gtk_widget_queue_draw(GTK_WIDGET(image1));

  /*------------*/

  gtk_widget_show_all (GTK_WIDGET(mainwindow)); /* ATTENTION */

  /* Lancer la boucle GTK */
  gtk_main ();

  /* La boucle GTK est terminee, detruire maintenant tous les widgets */
  g_object_unref (G_OBJECT (builder));

  /* Fin normale du programme */
  return 0;
}

/* Trace de cercle selon l'algorithme d'Andres (qui fut mon
 * professeur de Geometrie Discrete pour l'Infographie) qui
 * est meilleur que l'algorithme de Bresenham (de 1977).
 *
 * Eric Andres: Discrete circles, rings and spheres. In
 * Computers & Graphics Journal, Volume 18, Number 5,
 * pp. 695-706, 1994
 */
void draw_circle_andres (GdkPixbuf *pixbuf, int x_centre, int y_centre, int rayon, guchar R, guchar G, guchar B) {
  int x = 0 ;
  int y = rayon ; 
  int d = rayon-1 ;

  while (y >= x) {
        gdkpixbuf_set_pixel_color (pixbuf, x_centre + x, y_centre + y, R, G, B) ;
        gdkpixbuf_set_pixel_color (pixbuf, x_centre + y, y_centre + x, R, G, B) ;
        gdkpixbuf_set_pixel_color (pixbuf, x_centre - x, y_centre + y, R, G, B) ;
        gdkpixbuf_set_pixel_color (pixbuf, x_centre - y, y_centre + x, R, G, B) ;
        gdkpixbuf_set_pixel_color (pixbuf, x_centre + x, y_centre - y, R, G, B) ;
        gdkpixbuf_set_pixel_color (pixbuf, x_centre + y, y_centre - x, R, G, B) ;
        gdkpixbuf_set_pixel_color (pixbuf, x_centre - x, y_centre - y, R, G, B) ;
        gdkpixbuf_set_pixel_color (pixbuf, x_centre - y, y_centre - x, R, G, B) ;

        if (d >= 2*x) {
	  d = d - 2*x - 1 ;
	  x++ ;
	}
        else if (d <= 2*(rayon-y)) {
	  d = d + 2*y - 1;
	  y-- ;
	}
	else {
	  d = d + 2*(y-x-1) ;
	  y--;
	  x++ ;
	}
  }
}
